/*
 *
 *	AUTHOR : Milfar Dean (milfar.dean@quickoffice.com).
 *
 */


//The Array that holds the functions mapped in the jsFunctionMappings.js file
var functionsMap = new Array();


/*
 *  executeCommand sends a message to the native code.
 *	
 *	command - the command to be executed.
 *	parameterStringInJsonFormat - parameters to be passed to the backend in JSON format. 
 */


function executeCommand(command, parameterStringInJsonFormat, callbackFunction )
{
	//alert(command + " :: " + callbackFunction + "::" + parameterStringInJsonFormat);
	if(command && callbackFunction)
	{
		if(window.javascriptHandler)
		{
			window.javascriptHandler.executeCommand(command, callbackFunction, parameterStringInJsonFormat);
		}
		else
		{
	        var messageString = "cmd="+command+"&cbFun="+callbackFunction;
	        if(parameterStringInJsonFormat)
			{
	            messageString += "&msg="+encodeURIComponent(parameterStringInJsonFormat);
	        }
	        else
			{
	            messageString += "&msg=''";
	        }
			window.location = "call?"+messageString;
		}
    }
	else
    {
		alert("ERROR :: Both command and callbackFunction are mandatory. " +
			  "If you do not expect a callback please pass the string 'none' for callbackFunction.");
    }
}

function postCommand(command, parameterStringInJsonFormat)
{
	executeCommand(command, parameterStringInJsonFormat, 'none');
}

function logError(errorString)
{
	var errorJson = '{error:"' + errorString + '"}';
	executeCommand('logError', errorJson, 'none');
}


/*
 *  mapCommandToCallbackFunction maps the command send by the native code to the javascript function.
 *	This function is called in jsFunctionMappings.js
 */

function mapCommandToCallbackFunction(command, callbackFunction)
{
    var funcArray = functionsMap[command];
	
	if(funcArray == null)
	{
		funcArray = new Array();
		functionsMap[command] = funcArray;
	}
	funcArray.push(callbackFunction);
}


/*
 *  replaceAll replace all occurances of a string with another string.
 */

function replaceAll(originalString, stringToReplace, newString)
{
    while(originalString.indexOf(stringToReplace) > -1)
	{
        originalString = originalString.replace(stringToReplace, newString);
    }
    return originalString;
} 

/*
 *  replaceAllSpecialCharacters replaces all occurances of all special characters recieved from the native code.
 */

function replaceAllSpecialCharacters(objectReceived){
    if(typeof objectReceived == 'string')
	{
        objectReceived = replaceAll(objectReceived, "&napos;", "'");
        objectReceived = replaceAll(objectReceived, "&nlbracket;", "[");
        objectReceived = replaceAll(objectReceived, "&nrbracket;", "]");
        objectReceived = replaceAll(objectReceived, "&nlbrace;", "{");
        objectReceived = replaceAll(objectReceived, "&nrbrace;", "}");
	objectReceived = replaceAll(objectReceived, "\\", "");
    }
    else if(objectReceived instanceof Array)
	{
        var numRows = objectReceived.length;
        for(var i = 0; i < numRows; i++)
		{
            if(objectReceived[i])
			{
                objectReceived[i] = replaceAllSpecialCharacters(objectReceived[i]);
            }
        }
    } 
    else if(objectReceived instanceof Object)
	{
        var aKey;
        for (aKey in objectReceived)
		{
            if(objectReceived[aKey])
			{
                if(objectReceived[aKey] instanceof Function)
				{
                    continue;
                }
                objectReceived[aKey] = replaceAllSpecialCharacters(objectReceived[aKey]);
            }
        }
    }
    return objectReceived;
}

/*
 *  trim removes white space from the beginning and end of a string.
 */

//
function trim(originalString)
{
    return originalString.replace(/^\s+|\s+$/g, '');
}



function handleResponse(command, parameterStringInJsonFormat)
{
	var paramsArray = null;
	if(parameterStringInJsonFormat)
	{
		try
		{
			paramsArray = JSON.parse(parameterStringInJsonFormat);
		}
		catch(e)
		{
			alert(e);
		}
	}
    
	try
	{
		if(command)
		{
			var functionsList = functionsMap[command];
			if(functionsList == null)
			{
				functionsList = new Array();
			}
			var numOfFunctions = functionsList.length;
			for(var i = 0; i < numOfFunctions; i++)
			{
				
				try
				{
					var functionToInvoke = functionsList[i](paramsArray);
				}
				catch(err)
				{
					logError("175 Command : "+command+" " +err);
				}
			}	
		}
    }
    catch(err)
	{
    	logError("182"+err);
    }
}